{-----------------------------------------------------------------------
*  TColorButton v1.1                                                   *
*                                                                      *
*    Copyright  1996 InforTech, Inc.  All Rights Reserved.            *
*    By: Steven Costa Martins                                          *
*    E-MAIL: steven@mail.telepac.pt                                    *
*    Home Page:                                                        *
*    http://www.geocities.com/SiliconValley/Park/7619/                 *
*                                                                      *
*    Last Modification: 27 Oct 1996, 17:20 PM                          *
------------------------------------------------------------------------
*  You are not allowed to redistribute this                            *
*  code or any part of it                                              *
------------------------------------------------------------------------}

unit Colorbtn;

interface

{$A+} { Word align data }
{$B-} { Complete Boolean Evaluation Directive }
{$D-} { Debug Information Directive }
{$L-} { Local Symbol Information Directive }
{$R-} { Range checking off }
{$S-} { Stack checking off }
{$T-} { We don't need (nor want) this type checking! }

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, ExtCtrls, DsgnIntf;

type
  (*
   *  Property AboutBox
   *)
  TAboutColorButtonProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

  (*
   *  ColorButton
   *)
  TColorButton = class(TCustomControl)
  private
    { Private declarations }
    FAbout: TAboutColorButtonProperty;
    FDown: Boolean;
    FDragging: Boolean;
    FColor: TColor;
    FState: TButtonState;
    FOptions: TCOlorDialogOptions;
    FCustomColors: TStringList;
    FOnColorChange: TNotifyEvent;
    FMargin: Byte;

    procedure SetStateColor(Value: TColor);
    procedure SetOptions(Value: TCOlorDialogOptions);
    procedure SetCustomColors(Value: TStringList);
    procedure SetMargin(Value: Byte);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure WMSize(var Message:TWMSize); message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Click; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property About: TAboutColorButtonProperty read FAbout write FAbout;
    property Color: TColor read FColor write SetStateColor;
    property Options: TColorDialogOptions read FOptions write SetOptions;
    property CustomColors: TStringList read FCustomColors write SetCustomColors;
    property Margin: Byte read FMargin write SetMargin default 1;

    property OnColorChange: TNotifyEvent read FOnColorChange write FOnColorChange;

    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property TabOrder;
    property Visible;

    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

(*
 * About property
 *)


procedure TAboutColorButtonProperty.Edit;
var
	msg: string;
const
	carriage_return = chr(13);
begin
	msg := 'ColorButton Component v1.1';
  AppendStr(msg, carriage_return);  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Copyright  1996 InforTech, Inc.  All Rights Reserved.');
  AppendStr(msg, carriage_return);  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Developed by:');
  AppendStr(msg, carriage_return);  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Steven Costa Martins');
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'E-Mail: steven@mail.telepac.pt');
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Home Page:');
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'http://www.geocities.com/SiliconValley/Park/7619/');
  AppendStr(msg, carriage_return);
  ShowMessage(msg);
end;

function TAboutColorButtonProperty.GetAttributes: TPropertyAttributes;
begin
 	Result := [paMultiSelect, paDialog, paReadOnly];
end;

function TAboutColorButtonProperty.GetValue: string;
begin
	Result := 'Click on...for About box';
end;


(*
 *  ColorButton
 *)

constructor TColorButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 45;
  Height := 25;
  FColor := clWhite;
  ParentFont := True;
  FCustomColors := TStringList.Create;
  FMargin := 1;
end;

destructor TColorButton.Destroy;
begin
  FCustomColors.Free;
  inherited Destroy;
end;

procedure TColorButton.SetStateColor(Value: TColor);
begin
  FColor := Value;
  Repaint;
end;

procedure TColorButton.SetOptions(Value: TCOlorDialogOptions);
begin
  FOptions := Value;
end;

procedure TColorButton.SetCustomColors(Value: TStringList);
begin
  FCustomColors.Assign(Value);
end;

procedure TColorButton.SetMargin(Value: Byte);
begin
	if Value in [0..2] then FMargin := Value
  else Value := FMargin;
  Invalidate;
end;

function Win95DrawButtonFace(Canvas: TCanvas; const Client: TRect;
  BevelWidth: Integer; Style: TButtonStyle; IsDown,
  IsFocused: Boolean): TRect;
var
  NewStyle: Boolean;
  R: TRect;
begin
  R := Client;
  with Canvas do
  begin
    Pen.Color := clBtnFace;
    Brush.Color := clBtnFace;
    Brush.Style := bsSolid;
    Rectangle(R.Left, R.Top, R.Right, R.Bottom);

    if IsFocused then
    begin
      InflateRect(R, -1, -1);
      Brush.Style := bsClear;
      Rectangle(R.Left, R.Top, R.Right, R.Bottom);
    end;

    InflateRect(R, -1, -1);
    if not IsDown then
      begin
        {the frame when button is up}
        Frame3D(Canvas, R, clBtnHighlight, clWindowFrame, BevelWidth);
        Frame3D(Canvas, R, clBtnFace, clBtnShadow, BevelWidth);
      end
    else
    begin
        {the frame when button is pressed}
        Frame3D(Canvas, R, clWindowFrame, clBtnHighlight, BevelWidth);
        Frame3D(Canvas, R, clBtnShadow, clBtnFace, BevelWidth);
    end;
  end;

  Result := Client;
  InflateRect(Result, -BevelWidth, -BevelWidth);
  if IsDown then OffsetRect(Result, 1, 1);
end;

procedure TColorButton.Paint;

var
  ColorWidth,
  ColorHeight,
  ColorTop,
  ColorLeft: Integer;

  TextLeft: Integer;
  R: TRect;
  CStr: array[0..255] of Char;

  PaintRect: TRect;
begin
  ColorWidth := Width -15;
  ColorHeight := Height -15;
  ColorTop := (Height div 2) - (ColorHeight div 2) -1;
  ColorLeft := (Width div 2) - (ColorWidth div 2) -1;

  { Draw the Win95 style button face }
  PaintRect := Win95DrawButtonFace(Canvas, Rect(0, 0, Width -1, Height -1), 1, bsWin31,
    FState in [bsDown, bsExclusive], False);

  with Canvas do
  begin
    { draw focus rect around caption if necessary }
    if Self.Focused then
    begin
	  	R := Bounds(3, 3, Self.Width -8, Self.Height -8);

      if FState = bsDown then OffsetRect(R, 1, 1);
      Canvas.DrawFocusRect(R);
    end;

    R := Bounds(ColorLeft - 1 * FMargin, ColorTop - 1 * FMargin,
    						ColorWidth + 2 * FMargin, ColorHeight + 2 * FMargin);

    if FState = bsDown then OffsetRect(R, 0, 1);

    { draw border rectangle }
    Pen.Color := clBlack;
    Rectangle(R.Left, R.Top, R.Right +1, R.Bottom +1);

    { draw the Color }
    OffsetRect(R, 1, 1);
    Dec(R.Right);
    Dec(R.Bottom);
    if Enabled then Brush.Color := FColor
    else Brush.Color := clBtnFace;
    FillRect(R);
  end;
end;

procedure TColorButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
	R: TRect;
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then
  begin
    { draw focus rect around caption }
    SetFocus;
    Canvas.Pen.Color := clSilver;
    R := Bounds(3, 3, Self.Width - 8, Self.Height - 8);
    Canvas.DrawFocusRect(R);

    if not FDown then
    begin
      FState := bsDown;
      Repaint;
    end;
    FDragging := True;
  end;
end;

procedure TColorButton.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewState: TButtonState;
begin
  inherited MouseMove(Shift, X, Y);
  if FDragging then
  begin
    if not FDown then NewState := bsUp;

    if (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight) then
      if FDown then NewState := bsUp else NewState := bsDown;
    if NewState <> FState then
    begin
      FState := NewState;
      Repaint;
    end;
  end;
end;

procedure TColorButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  inherited MouseUp(Button, Shift, X, Y);
  if FDragging then
  begin
    FDragging := False;
    DoClick := FDown and (X >= 0) and (X < ClientWidth) and (Y >= 0) and
      (Y <= ClientHeight);
    FState := bsUp;
    Repaint;
    if DoClick then Click;
  end;
end;

procedure TColorButton.Click;
var
  ColorDlg: TColorDialog;
begin
  Try
    ColorDlg := TColorDialog.Create(Self);
    With ColorDlg do
    begin
      Color := Self.Color;
      Options := FOptions;
      CustomColors := Self.CustomColors;
    	if Execute then
      begin
	      FColor := ColorDlg.Color;
        if Assigned(FOnColorChange) then FOnColorChange(Self);
      end;
    end;
    inherited Click;
  Finally
    ColorDlg.Free;
  End; {Try}
end;

{-------------------------------------------------------------------------}
{VCL Messages}

procedure TColorButton.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TColorButton.CMSysColorChange(var Message: TMessage);
begin
  Invalidate;
end;

procedure TColorButton.WMSize(var Message:TWMSize);
begin
  Invalidate;
end;

procedure TColorButton.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  Invalidate;
end;

procedure TColorButton.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('InforTech', [TColorButton]);
  {Register the about box property editor}
  RegisterPropertyEditor(TypeInfo(TAboutColorButtonProperty),
      TColorButton, 'ABOUT', TAboutColorButtonProperty);

end;

end.


